<?php

class KnyttBinReader
{
	private $rootDir = '';
	private $binSize = 0;
	private $files = null;
	
	//Destructor
	function __destruct()
	{
		unset($files);
	}
	
	//Read .knytt.bin
	public function Read($file, $skipSongs=true)
	{
		$this->rootDir = '';
		$this->binSize = 0;
		$this->files = null;
		
		//File doesn't exist, quit
		if(!file_exists($file)) return false;
		
		//Read size
		$this->binSize = filesize($file);
		
		//Open file for reading
		$handle = fopen($file, 'r');
		
		//Check for indicator 'NF'
		if(fread($handle, 2) != 'NF') return false;
		
		//Get root directory
		while(ord($chr = fread($handle, 1)))
			$this->rootDir .= $chr;
		
		//Skip number of files (incorrect anyway?)
		$fileCount = fread($handle, 4);
		
		//Loop through file
		while(ftell($handle) < $this->binSize)
		{
			//Skip NF
			fread($handle, 2);
			
			$filePath = '';
			//Add character to file path until we hit a terminator
			while(ord($chr = fread($handle, 1)))
				$filePath .= ($chr == '\\') ? '/' : $chr;
			
			//Read size of following file
			$sizeBytes = fread($handle, 4);
			$this->files[$filePath]['size'] = ord($sizeBytes{0}) + ord($sizeBytes{1})*256 + ord($sizeBytes{2})*65536 + ord($sizeBytes{3})*16777216;
			//Read file
			if(!$skipSongs)
				$this->files[$filePath]['data'] = fread($handle, $this->files[$filePath]['size']);
			else
			{
				if(substr($filePath, 0, 5)!='Music' && substr($filePath, 0, 8)!='Ambiance')
					$this->files[$filePath]['data'] = fread($handle, $this->files[$filePath]['size']);
				else
					$this->files[$filePath]['data'] = fseek($handle, $this->files[$filePath]['size'], SEEK_CUR);
			}
		}

		//Done. Close file
		fclose($handle);
		
		return $fileCount;
	}
	
	//Save
	public function SaveFile($file, $output='')
	{
		//File not found, quit
		if(!isset($this->files[$file])) return false;
		
		//If no output path is specified, use file name
		if(!$output) $output = basename($file);
		
		//Write file
		$handle = fopen($output, 'w');
		fwrite($handle, $this->files[$file]['data']);
		fclose($handle);
		
		return true;
	}
	
	//Get root directory
	public function GetRoot()
	{
		return $this->rootDir;
	}
	
	//Get file
	public function GetFile($file)
	{
		//File not found, quit
		if(!isset($this->files[$file])) return false;
		
		return $this->files[$file]['data'];
	}

	//Get file size
	public function GetFileSize($file)
	{
		//File not found, quit
		if(!isset($this->files[$file])) return 0;
		
		return $this->files[$file]['size'];
	}

	//Get list of filenames
	public function GetFileList()
	{
		//No files read yet, quit
		if(!isset($this->files)) return false;
		
		return array_keys($this->files);
	}

	//Get list of filenames that metch regexp, e.g. /.png$/i
	public function GetFilesRegexp($regexp)
	{
		//No files read yet, quit
		if(!isset($this->files)) return false;
		
		$return = array();
		$keys = array_keys($this->files);
		
		//Loop through all files and check
		foreach($keys as $k => $v)
		{
			if(preg_match($regexp, $v))
				$return[$k] = $v;
		}
		
		return $return;
	}
	
	public function ReadINI($file)
	{
		//File not found, quit
		if(!isset($this->files[$file])) return false;

		$lines	= explode("\n", $this->files[$file]['data']);
		$return	= array();
		$inSect	= false;
		
		foreach($lines as $line)
		{
				$line =	trim($line);
				if(!$line || $line[0]=="; ")
					continue;
				if($line[0] == '[' && $endIdx = strpos($line, ']'))
				{
					$inSect	= substr($line, 1, $endIdx-1);
					continue;
				}
				if(!strpos($line, '='))
					continue;	
				$tmp = explode('=', $line, 2);
				if($inSect)
					$return[$inSect][trim($tmp[0])]	= ltrim($tmp[1]);
				else
					$return[trim($tmp[0])] = ltrim($tmp[1]);
		}
		
		return $return;
	}
}
?>